# Copyright (c) John DeVries
# All rights reserved.

# This code is licensed under the MIT License.

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files(the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions :

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from pathlib import Path


SECRET_KEY = "foobar"


INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.sites",
]


REST_FRAMEWORK = {
    "DEFAULT_AUTHENTICATION_CLASSES": [
        "rest_framework.authentication.SessionAuthentication"
    ],
    "DEFAULT_CONTENT_NEGOTIATION_CLASS": "htmx_rest.negotiator.HtmxContentNegotiator",
    "DEFAULT_RENDERER_CLASSES": [
        "rest_framework.renderers.JSONRenderer",
        "htmx_rest.renderers.HTMXPartialTemplateRenderer",
    ],
}

DATABASES = {"default": {"ENGINE": "django.db.backends.sqlite3", "NAME": "mem_db"}}


TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [Path(Path(__file__).parent, "templates")],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]



# Copyright (c) John DeVries
# All rights reserved.

# This code is licensed under the MIT License.

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files(the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions :

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from unittest.mock import patch

from rest_framework.test import APIRequestFactory
from rest_framework.views import APIView
from rest_framework.renderers import (
    TemplateHTMLRenderer,
    JSONRenderer,
    BrowsableAPIRenderer,
)
import pytest

from htmx_rest.negotiator import HtmxContentNegotiator


@pytest.fixture
def get_request():
    req = APIRequestFactory().get("/", HTTP_ACCEPT="text/html")
    return APIView().initialize_request(req)


@pytest.fixture
def test_renderers():
    """This particular test order is important. The TemplateHTMLRenderer is
    what we want for rendering htmx partials. The JSONRenderer is the
    default, and the BrowsableAPIRenderer is the default for content type
    of text/html"""
    return (
        JSONRenderer(),
        BrowsableAPIRenderer(),
        TemplateHTMLRenderer(),
    )


@pytest.fixture
def default_test_renderers():
    return (
        JSONRenderer(),
        BrowsableAPIRenderer(),
    )


def test_select_renderer_returns_htmx_partial(get_request, test_renderers):
    with patch("htmx_rest.negotiator.is_hx_request", return_value=True):
        neg = HtmxContentNegotiator()
        renderer_cls, content_type = neg.select_renderer(get_request, test_renderers)
        assert isinstance(renderer_cls, TemplateHTMLRenderer)
        assert content_type == "text/html"


def test_select_renderer_returns_browsable_api_for_normal_request(
    get_request, test_renderers
):
    with patch("htmx_rest.negotiator.is_hx_request", return_value=False):
        neg = HtmxContentNegotiator()
        renderer_cls, content_type = neg.select_renderer(get_request, test_renderers)
        assert isinstance(renderer_cls, BrowsableAPIRenderer)
        assert content_type == "text/html"


def test_raises_value_error_if_no_html_renderer_present(
    get_request, default_test_renderers
):
    with patch("htmx_rest.negotiator.is_hx_request", return_value=True):
        neg = HtmxContentNegotiator()
        with pytest.raises(ValueError) as exc:
            neg.select_renderer(get_request, default_test_renderers)
            assert "no renderer_class capable of rendering html" in str(exc.value)



# Copyright (c) John DeVries
# All rights reserved.

# This code is licensed under the MIT License.

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files(the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions :

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from types import SimpleNamespace
from unittest.mock import patch

import pytest

from .test_negotiator import get_request
from htmx_rest.renderers import HTMXPartialTemplateRenderer


@pytest.fixture
def prepared_renderer():
    renderer = HTMXPartialTemplateRenderer()
    renderer.template_name = "tests/partial.html"
    return renderer


@pytest.fixture
def expected_html():
    return """<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>HTMX Partial</title>
  </head>
  <body>
    <h1>Htmx template:</h1>
    <div style="border: 1px solid black; padding: 10px; margin: 10px;">
      <div>
  <h1>Hello, world!</h1>
</div>

    </div>
  </body>
</html>"""


@pytest.fixture
def expected_partial():
    return """<div>
  <h1>Hello, world!</h1>
</div>
"""


def test_normal_returns_partial(prepared_renderer, get_request, expected_html):
    res = prepared_renderer.render(
        {"message": "Hello, world!"},
        "text/html",
        {
            "view": "foo view",
            "request": get_request,
            "response": SimpleNamespace(
                exception=False, template_name=prepared_renderer.template_name
            ),
        },
    )
    assert res == expected_html


def test_hx_request_returns_partial(prepared_renderer, get_request, expected_partial):
    with patch("htmx_rest.renderers.is_hx_request", return_value=True):
        res = prepared_renderer.render(
            {"message": "Hello, world!"},
            "text/html",
            {
                "view": "foo view",
                "request": get_request,
                "response": SimpleNamespace(
                    exception=False, template_name=prepared_renderer.template_name
                ),
            },
        )
        assert res == expected_partial



# Copyright (c) John DeVries
# All rights reserved.

# This code is licensed under the MIT License.

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files(the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions :

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from rest_framework.test import APIRequestFactory
import pytest

from htmx_rest.util import is_hx_request

# last spot: https://docs.pytest.org/en/latest/how-to/fixtures.html#parametrizing-fixtures
@pytest.fixture(scope="module", params=(True, False))
def request_(request):
    return APIRequestFactory().get("/", HTTP_HX_REQUEST="true" if request.param else "")


def test_is_hx_request(request_):
    answer = is_hx_request(request_)
    assert answer == (request_.META.get("HTTP_HX_REQUEST") == "true")



# Copyright (c) John DeVries
# All rights reserved.

# This code is licensed under the MIT License.

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files(the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions :

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.


from rest_framework.renderers import TemplateHTMLRenderer

from .util import is_hx_request


# TODO: it would be awesome if there was a cool BrowsableAPIRenderer 2.0
# for this drf-htmx integration where you can also browse your views as
# web components alongside the json.


DUMMY_TEMPLATE = """<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>HTMX Partial</title>
  </head>
  <body>
    <h1>Htmx template:</h1>
    <div style="border: 1px solid black; padding: 10px; margin: 10px;">
      {}
    </div>
  </body>
</html>"""


class HTMXPartialTemplateRenderer(TemplateHTMLRenderer):
    """Will send the partial page when the `Hx-Request` header is set to true.
    Otherwise, it will wrap the partial content in a dummy html template.
    """

    def render(self, *a, **kw):
        content = super().render(*a, **kw)

        if is_hx_request(a[2]["request"]):
            return content

        return DUMMY_TEMPLATE.format(content)



# Copyright (c) John DeVries
# All rights reserved.

# This code is licensed under the MIT License.

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files(the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions :

# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from rest_framework.negotiation import DefaultContentNegotiation
from rest_framework.renderers import BrowsableAPIRenderer

from .util import is_hx_request


class HtmxContentNegotiator(DefaultContentNegotiation):
    def select_renderer(self, request, renderers, format_suffix=None):
        """In the presence of a `HX-Request: true` header, return html content,
        and avoid using the BrowsableAPIRenderer, which is the default for
        html content types.
        """
        if is_hx_request(request):
            for renderer in renderers:
                if "text/html" in renderer.media_type and not isinstance(
                    renderer, BrowsableAPIRenderer
                ):
                    return (renderer, renderer.media_type)
            raise ValueError(
                "no renderer_class capable of rendering html (not including "
                "the BrowsableAPIRenderer) was provided."
            )

        return super().select_renderer(request, renderers, format_suffix)
